/*
    Driver data type definition.
    Copyright (C) 2011 2014 2015 2016 2018  Alessandro Accattini

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 *****************************************************************************
 *
 *
 * $Header$
 *
 *
 *
 * REVISION HISTORY:
 * $Log$
 */

#ifndef __DRV_TYPE_H
#define __DRV_TYPE_H	1


#include "datatype.h"				// special data types

/***************************************************************************/
/* define pubblici */

/***************************************************************************/
/* macro pubbliche */

#define CHIP_SET_NAME(Name)	Name##_Driver
#define CHIP_SET_EXPORT(Name)	extern const DrvInt_t CHIP_SET_NAME(Name)
//#define CHIP_SET(Vname,Dname,Map,Init,cw,dw,dr,bw,br,bs,bn)	const DrvInt_t CHIP_SET_NAME(Vname)={Dname,__FILE__,Map,Init,cw,dw,dr,bw,br,bs,bn}
//#define CHIP_SET(Vname,Dname,Map,Init,cw,dw,dr,bw,br,bs,bn)	const DrvInt_t Vname##_Driver = {Dname,__FILE__,Map,Init,cw,dw,dr,bw,br,bs,bn}
#define CHIP_SET(Vname,Dname,Map,Init,cw,dw,dr,bw,br,bs,bn)	const DrvInt_t PIPPO

/***************************************************************************/
/* strutture pubbliche */

/** \brief  Driver init method.
 *
 *  This method is used to initialize the driver itself, usually it
 *  sets the initial state of the data bus and eventually initialize the
 *  device to place it in a well known state.
 *
 *  It is called passing to it a pointer to the driver descriptor itself and
 *  a variable number of parameters.
 */
typedef int (*BusInit_t)   (void *Chipset,int argc,char **argv);


/** \brief  Command send method.
 *
 *  It is used to send commands to the device. This method is usually not
 *  implemented and can be seen as a parallel channel to control the device.
 */
typedef int (*CmdSend_t)   (		 const Byte_t *Cmd,	int Len);


/** \brief  Data write method as character device.
 *
 *  This method is used to send data to the device as caracter device: a
 *  string of bytes of arbitrary lenght can be sent to the device.
 */
typedef int (*CharWrite_t) (Addr_t Addr, const char *Text,	int Len);


/** \brief  Data read method as character devide.
 *
 *  This method is used to read data from the device as caracter device: a
 *  string of bytes of arbitrary lenght can be retrieved from the device.
 */
typedef int (*CharRead_t)  (Addr_t Addr, char *Text,		int Len);


/** \brief  Data write method as block device.
 *
 *  This method is used to send data to the device as block device: usually
 *  Len is the size of a single data block.
 */
typedef int (*BlockWrite_t)(Addr_t Addr, const Byte_t *Data,	int Len);


/** \brief  Data read method as block device.
 *
 *  This method is used to read data from the device as block device: usually
 *  Len is the size of a single data block.
 */
typedef int (*BlockRead_t) (Addr_t Addr, Byte_t *Data,		int Len);



/** \brief  Device driver structure.
 *
 *  Each device driver is represented by a data structure containing driver
 *  name, file that implements the driver itself, some data on required
 *  hardware link and all the methods exported by the driver.
 *
 *  The interface is the same for all device drivers.
 *
 *  Basically each device can be addressed as block device or character
 *  device (or both in some strange case). Each driver exports some mothods
 *  to access to the device itself.
 *  Not all possible methot must be implemented: each driver implements only
 *  required method to access to the full device or a part of it if it is not
 *  possible to access the full device at once: some particular device can be
 *  devided in two or more subdevices to simplify acess methods.
 *
 *  For example a microcontroller with embedded flash and eeprom memory cen
 *  be seen as two different devices at least: a flash memory, accessed as
 *  block device; an eeprom memory, accessed as block device; a bounch of
 *  fuses, probably accessed ad character device.
 *  All these methods cannot be included in a single device driver, we have
 *  to implement 3 differents device derivers for this single microcontroller.
 */
typedef struct
{
  const char	*DrvName;	/**< Driver name, this name must be unique. */
  const char	*DrvFile;	/**< Driver implementation file name. */
  const char	*LinkMap;	/**< Hardware link ascii-art. */

  BusInit_t	BusInit;	/**< Bus init method, this method is used to
				     initialize the driver itself. */
  CmdSend_t	CmdSend;	/**< Method used to send commands. */
  CharWrite_t	CharWrite;	/**< Method used to write data to the device
				     as character device. */
  CharRead_t	CharRead;	/**< Method used to read data from the device
				     as character device. */
  BlockWrite_t	BlockWrite;	/**< Method used to write data to the device
				     as block device. */
  BlockRead_t	BlockRead;	/**< Method used to read data from the device
				     as block device. */

  int		BlockSize;	/**< Single block size or lenght of a single
				     row. */
  int		BlocksNum;	/**< Total number of blocks or rows number. */
}DrvInt_t;

/***************************************************************************/
/* costanti pubbliche */

/***************************************************************************/
/* variabili pubbliche */

/***************************************************************************/
/* funzioni pubbliche */

/***************************************************************************/
#endif

/* $Source$ */

