/*
    I/O level handling functions.
    Copyright (C) 2010 2013 2015 2016 2018 2020  Alessandro Accattini

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 *****************************************************************************
 *
 *
 * $Header$
 *
 *
 *
 * REVISION HISTORY:
 * $Log$
 */

/**  \file    io.h
     \author  Alessandro Accattini
     \date    17 Gen 2020

     \brief   Basic I/O level handling functions.

     I/O level is the lowest available level, at this level it is possible to
     handle single I/O bits (aka single data lines).
 */


#ifndef __IO_H
#define __IO_H	1


#include <stdint.h>

/***************************************************************************/
/* define pubblici */

  // pin direction
#define INPUT			0		// pin in ingresso
#define OUTPUT			1		// pin in uscita


#define IO_BUS_WIDTH		8		// larghezza del bus in bit

/***************************************************************************/
/* macro pubbliche */

/***************************************************************************/
/* strutture pubbliche */

/** \brief  Bus data type.
 *
 *  Hardware bus is represented by 8 data lines; each line can be configured
 *  as input or output indipendently from the others. We need a data type to
 *  represent the full bus at once.
 */
typedef uint8_t			BusData_t;

/***************************************************************************/
/* costanti pubbliche */

/***************************************************************************/
/* variabili pubbliche */

/***************************************************************************/
/* funzioni pubbliche */

int IO_Open(int Vendor,int Product,int HardIfaces);
  /**< \brief  Opens and initilizes the hardware interface.
       \param  Vendor  vendor ID.
       \param  Product product ID.
       \param  HardIfaces  ignored.
       \return 0 on success.

       Searches for hardware interface and initializes low hardware level,
       must be called before any other function.
       If this function ends with success all has gone well, in other case
       hardware interface is not connected or cannot be found. An error
       message is printed on stderr.
   */

int IO_Close(void);
  /**< \brief  Close and release the hardware interface.
       \return 0 on success.

       Close the hardware interface and free all resources;
       after this call is not possible to call any other function.
   */


int IO_SetBaudRate(int Baud);
  /**< \brief  Change the hardware baudrate.
       \param  Baud  the new baudrate.
       \return 0 on success.

       Changing baudrate can influence the bus speed.
   */


BusData_t IO_DirMaskRead(void);
  /**< \brief  Reads back the full port direction register.
       \return The bus direction mask.

       Used to read back the full bus direction register: each bit in
       the register defines direction of the corresponding data bit.
   */

int IO_DirMaskWrite(BusData_t DirMask);
  /**< \brief  Sets the full port direction register.
       \param DirMask    new direction mask.
       \return           0 on success.

       Overwrite the full bus direction register.
   */


BusData_t IO_DirMaskBitRead(uint16_t Bit);
  /**< \brief  Reads back a single bit of direction register.
       \param Bit    Bit number (starting from 0).
       \return       Bit direction.

       Reads back a single bit of direction register, bit value is right
       aligned so that returned value is always 0 or 1.
   */

int IO_DirMaskBitWrite(uint16_t Bit,BusData_t Dir);
  /**< \brief  Writes a single bit of direction register.
       \param Bit    Bit number (starting from 0).
       \param Dir    New bit direction (0 or 1).
       \return       0 on success.

       Used to change the direction of a single I/O bit without affecting all
       the others.
   */


BusData_t IO_OutDataRead(void);
  /**< \brief  Reads back the output data register.
       \return The output data register value.

       Reads back the content of the output data register, to
       read input data register use \ref IO_InDataRead() instead.
   */

BusData_t IO_InDataRead(void);
  /**< \brief  Reads the input data register.
       \return The input data register value.

       All input data lines are read sincronously.
   */

void IO_OutDataWrite(BusData_t Data);
  /**< \brief  Writes a new value to the output data register.
       \param Data    New register value.

       Writes a new value to the output data register.
       All output data lines update sincronously.
   */


BusData_t IO_BitRead(uint16_t Bit);
  /**< \brief  Reads a single input bit value.
       \param Bit     Bit number.
       \return        Bit value (0 or 1).

       Returns a sincronously updated input bit value if 
       \p Bit is configured as input; returns the last set bit value if
       \p Bit is configured as output (i.e. the last value written with
       \ref IO_BitWrite()).
   */

void IO_BitWrite(uint16_t Bit,BusData_t Value);
  /**< \brief  Writes a single output bit value.
       \param Bit     Bit number (starting from 0).
       \param Value   new bit value.

       Writes a single output bit value.
       Writing to a bit configured as input do nothing.
   */

/***************************************************************************/
#endif

/* $Source$ */

