/*
    PCF8833 based LCD display driver.
    Copyright (C) 2012 2013 2016  Alessandro Accattini

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 *****************************************************************************
 *
 *
 * $Header$
 *
 *
 *
 * REVISION HISTORY:
 * $Log$
 */


/***************************************************************************/
/* include vari */

#define __PCF8833_C
#include "pcf8833.h"				// functions prototypes

#include "io.h"					// I/O level
#include "spi.h"				// SPI channel
#include "font.h"				// font dei caratteri

/***************************************************************************/
/* define interni alla libreria */

#define DATA			0x0100		// data byte
#define COMMAND			0x0000		// command byte

  // display commands
#define NOP 	0x00 		// nop
#define SWRESET 0x01 		// software reset
#define BSTROFF 0x02 		// booster voltage OFF
#define BSTRON 	0x03 		// booster voltage ON
#define RDDIDIF 0x04 		// read display identification
#define RDDST 	0x09 		// read display status
#define SLEEPIN 0x10 		// sleep in
#define SLEEPOUT 0x11 		// sleep out
#define PTLON 	0x12 		// partial display mode
#define NORON 	0x13 		// display normal mode
#define INVOFF 	0x20 		// inversion OFF
#define INVON 	0x21 		// inversion ON
#define DALO 	0x22 		// all pixel OFF
#define DAL 	0x23 		// all pixel ON
#define SETCON 	0x25 		// write contrast
#define DISPOFF 0x28 		// display OFF
#define DISPON 	0x29 		// display ON
#define CASET 	0x2A 		// column address set
#define PASET 	0x2B 		// page address set
#define RAMWR 	0x2C 		// memory write
#define RGBSET 	0x2D 		// colour set
#define PTLAR 	0x30 		// partial area
#define VSCRDEF 0x33 		// vertical scrolling definition
#define TEOFF 	0x34 		// test mode
#define TEON 	0x35		// test mode
#define MADCTL 	0x36 		// memory access control
#define SEP 	0x37 		// vertical scrolling start address
#define IDMOFF 	0x38 		// idle mode OFF
#define IDMON 	0x39 		// idle mode ON
#define COLMOD 	0x3A 		// interface pixel format
#define SETVOP 	0xB0 		// set Vop
#define BRS 	0xB4 		// bottom row swap
#define TRS 	0xB6 		// top row swap
#define DISCTR 	0xB9 		// display control
#define DOR 	0xBA 		// data order
#define TCDFE 	0xBD 		// enable/disable DF temperature compensation
#define TCVOPE 	0xBF 		// enable/disable Vop temp comp
#define EC	0xC0 		// internal or external oscillator
#define SETMUL 	0xC2 		// set multiplication factor
#define TCVOPAB 0xC3 		// set TCVOP slopes A and B
#define TCVOPCD 0xC4 		// set TCVOP slopes c and d
#define TCDF 	0xC5 		// set divider frequency
#define DF8COLOR 0xC6 		// set divider frequency 8-color mode
#define SETBS 	0xC7 		// set bias system
#define RDTEMP 	0xC8 		// temperature read back
#define NLI 	0xC9 		// n-line inversion
#define RDID1 	0xDA 		// read ID1
#define RDID2 	0xDB 		// read ID2
#define RDID3 	0xDC 		// read ID3


#define SET_COLUMN_ADDRESS	0x2a
#define SET_ROW_ADDRESS		0x2b
#define SLEEP_OUT		0x11
#define DISPON			0x29
#define BSTRON			0x03
#define MEMORY_WRITE		0x2c
#define DISPLAY_NOP		0x00

#define OLED_COLS		21		// numero di colonne testo
#define OLED_ROWS		16		// numero di righe testo
#define FONT_W			6		// larghezza char
#define FONT_L			8		// altezza char

#define BYTES_LINE		64		// bytes per riga nel display
#define PIXEL_ROWS		128		// righe totali del display

#define FONT_COLOR		0x0f		// colore testo

#define INIT_DATA		0x00		// valore di inizializzazione

/***************************************************************************/
/* macro interne alla libreria */

#define SetCS(v)	IO_BitWrite(3,v)
#define SetRES(v)	IO_BitWrite(4,v)

/***************************************************************************/
/* strutture contenute all'interno della libreria */

/***************************************************************************/
/* costanti interne alla libreria */

//static const unsigned char	BusMap[8]={2,4,0,1,3,5,6,7};
static const unsigned char	DispSleepOut[]=	{SLEEP_OUT};
static const unsigned char	DispOn[]=	{DISPON};
static const unsigned char	DispBstrOn[]=	{BSTRON};

static const unsigned char	RGB_8bit_Palette[]={
		0, 4, 9,13,18,22,27,31, 0, 0, 0, 0, 0, 0, 0, 0,
		0, 9,18,27,36,45,54,63, 0, 0, 0, 0, 0, 0, 0, 0,
		0,10,21,31, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};

/***************************************************************************/
/* variabili interne alla libreria */

/***************************************************************************/
/* funzioni interne alla libreria */

static void CmdWrite(unsigned char Cmd,unsigned char Mode);
  /* Invia un command byte al display driver. */

static void DataWrite(unsigned char Data);
  /* Invia un data byte al display driver. */

static void MultiDataWrite(const unsigned char Data[],int DataLen);
  /* Invia un array di data bytes al display driver. */

static void SetWindow(unsigned char x1,unsigned char y1,
			unsigned char x2,unsigned char y2);
  /* Imposta l'area in cui sara' aggiornata l'immagine. */

static void PutChar(int Ascii);
  /* Stampa un char alla posizione attuale del cursore, il cursore avanza
     di una posizione, raggiunta la fine della riga il cursore passa alla
     riga successiva, ragginta la fine del video riparte della prima riga. */

/***************************************************************************/
/* funzioni */

static void CmdWrite(unsigned char Cmd,unsigned char Mode)
{
  /* Invia un command byte al display driver. */



  SetCS(0);					// abbassa CS
  SPI_DataExchange((unsigned long int)Cmd | COMMAND);
  if(Mode == CLOSE_DEVICE)			// se e' un byte singolo
    SetCS(1);					// alza CS
  return;					// termina
}



static void DataWrite(unsigned char Data)
{
  /* Invia un data byte al display driver. */
  
  
  
  SetCS(0);					// abbassa CS
  SPI_DataExchange((unsigned long int)Data | DATA);
  SetCS(1);					// alza CS
  return;					// termina
}



static void MultiDataWrite(const unsigned char Data[],int Len)
{
  /* Invia un array di data bytes al display driver. */
  
  
  
  SetCS(0);					// abbassa CS
//  SPI_DataExchange(MEMORY_WRITE);		// invia il comando
  while(Len)
  {
    SPI_DataExchange((unsigned long int)*Data | DATA);
    Data++;					// next data byte
    Len--;					// scala il contatore
  }
//  SPI_DataExchange(DISPLAY_NOP);		// termina il data stream
  SetCS(1);					// alza CS
  return;					// termina
}




static void SetWindow(unsigned char x1,unsigned char y1,
			unsigned char x2,unsigned char y2)
{
  /* Imposta l'area in cui sara' aggiornata l'immagine. */



  unsigned char	CmdBuf[2];			// comando spedito



  // imposta le colonne della finestra
  CmdBuf[0]=x1;
  CmdBuf[1]=x2;
  CmdWrite(CASET,LEAVE_IT_OPEN);		// imposta le colonne
  MultiDataWrite(CmdBuf,2);
  
  // imposta le righe della finestra
  CmdBuf[0]=y1;
  CmdBuf[1]=y2;
  CmdWrite(PASET,LEAVE_IT_OPEN);		// imposta la pagina (righe)
  MultiDataWrite(CmdBuf,2);
  return;					// termina
}



static void PutChar(int Ascii)
{
  /* Stampa un char alla posizione attuale del cursore, il cursore avanza
     di una posizione, raggiunta la fine della riga il cursore passa alla
     riga successiva, ragginta la fine del video riparte della prima riga. */



  const unsigned char	*Font;			// char shape
  int			i,j;
  int			k=0;
  unsigned char		Shape;			// row shape
  unsigned char		Data[3*8];		// display data array



  switch(Ascii)
  {
    case '\n':
      FontNewLine();				// va a capo
    break;
    case '\r':
      CursorX=0;				// inizio riga
    break;
    default:
      Font=FontGetChar(Ascii);			// punta alla char shape
      for(i=0;i<8;i++)				// cicla le righe
      {
        Shape=*Font;				// legge un byte dal font
	Font++;					// punta al successivo
        for(j=0;j<3;j++)			// cicla le colonne
	{
	  Shape<<=2;				// sposta avanti 2 posti
	  Data[k]=0x00;				// 2 pixel neri
	  if(Shape & 0x80)
	    Data[k]|=0xf0;
	  if(Shape & 0x40)
	    Data[k]|=0x0f;
	  k++;					// next byte in data array
	}
      }
      SetWindow(CursorX,CursorY,CursorX+2,CursorY+7);
      CmdWrite(MEMORY_WRITE,LEAVE_IT_OPEN);	// RAM write
      MultiDataWrite(Data,3*8);			// riempie la finestra
      CmdWrite(NOP,CLOSE_DEVICE);		// data end
      CursorX+=3;
      if(CursorX > 60)
        FontNewLine();				// va a capo
    break;
  }
  return;					// termina
}



int PCF8833_InitBus(void *Chipset,int argc,char **argv)
{
  /* Imposta lo stato iniziale del bus e inizializza il display, ritorna 0
     in caso di successo, 1 altrimenti. */



  // inizializza il bus
  if(SPI_InitBus(SPI_CPOL_LOW|SPI_FIRST_EDGE|SPI_MSB_FIRST,9))
    return 1;					// termina con fallimento

  // imposta direzione e stato dei bit non SPI
  IO_DirMaskSet(0x18);				// bit 3 e 4 in uscita
  SetCS(1);					// alza CS
  
  // display hardware reset
  SetRES(0);					// abbassa RES
  SetRES(1);					// alza RES

  // display power up sequence
  CmdWrite(SLEEPOUT,CLOSE_DEVICE);
  CmdWrite(MADCTL,LEAVE_IT_OPEN);
  DataWrite(0x20);
  CmdWrite(SETCON,LEAVE_IT_OPEN);
  DataWrite(0x30);
  CmdWrite(DISPON,CLOSE_DEVICE);
  CmdWrite(COLMOD,LEAVE_IT_OPEN);
  DataWrite(2);					// 8 bit color mode

  CmdWrite(RGBSET,LEAVE_IT_OPEN);
  MultiDataWrite(RGB_8bit_Palette,16*3);
  CmdWrite(NOP,CLOSE_DEVICE);

  // clear display
  //SetWindow(0,0,63,127);			// tutto il display
  return 0;					// termina con successo
}



int PCF8833_CmdWrite(const unsigned char Cmd[],int Len)
{
  /* Invia un comando composito al display driver. */



  if(Len > 1)
  {
    CmdWrite(*Cmd++,LEAVE_IT_OPEN);
    MultiDataWrite(Cmd,Len-1);
  }
  else
  {
    CmdWrite(*Cmd,CLOSE_DEVICE);
  }
  return 0;					// termina con successo
}



int PCF8833_CopyFrame(unsigned long int Addr,const unsigned char *Frame,int Len)
{
  /* Copia il frame buffer specificato sul diplay partendo dall'alto. */
  
  
  
  SetWindow(0,0,63,127);			// tutto il display
  MultiDataWrite(Frame,Len);			// copy frame buffer
  return 0;					// termina con successo
}



int PCF8833_TextWrite(unsigned long int Addr,const char *Str,int Len)
{
  /* Stampa una stringa a video come se fosse un display alfanumerico. */
  
  
  
  while(Len--)					// fino al trminatore
    PutChar(*Str++);				// stampa un char
  return 0;					// termina con successo
}

/* $Source$ */

