/*
    SPI bus master implementation.
    Copyright (C) 2010 2011 2016 2018 2020  Alessandro Accattini

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 *****************************************************************************
 *
 *
 * $Header$
 *
 *
 *
 * REVISION HISTORY:
 * $Log$
 */


/***************************************************************************/
/* include vari */

#define __SPI_C
#include "spi.h"				// functions prototypes

#include "io.h"					// hardware control
#include "trace.h"				// function call trace

/***************************************************************************/
/* define interni alla libreria */

/**
 *  /brief SPI pins mapping.
 *
 *  SPI bus emulation uses a pre-defined set of pins, these pins are
 *  dedicated to SPI bus and cannot be used by I/O operations.
 */
#define MOSI		0		///< D0 is used as MOSI.
#define MISO		1		///< D1 is used as MISO.
#define CLK		2		///< D2 il used as CLK.

/***************************************************************************/
/* macro interne alla libreria */

#define SetMosi(v)	IO_BitWrite(MOSI,v)
#define GetMiso()	IO_BitRead(MISO)
#define SetClk(v)	SpiClk=v; IO_BitWrite(CLK,SpiClk)
#define ToggleClk()	SpiClk^=1; IO_BitWrite(CLK,SpiClk)

/***************************************************************************/
/* strutture contenute all'interno della libreria */

/***************************************************************************/
/* costanti interne alla libreria */

/***************************************************************************/
/* variabili interne alla libreria */

static unsigned int	Nbits=8;		// lunghezza singola word
static unsigned char	SpiMode=0x00;		// operating mode
static BusData_t	SpiClk=0;		// clock idle level

/***************************************************************************/
/* funzioni interne alla libreria */

/***************************************************************************/
/* funzioni */

int SPI_InitBus(unsigned int Mode,unsigned int BitPerWord)
{
  /* Imposta la direzione e lo stato iniziale delle linee del bus,
     ritorna 0 in caso di successo. */
  
  

  FUNCTION_ENTER();

  SpiMode=Mode;					// salva la modalita op.
  Nbits=BitPerWord;				// salva la dim. di una word
  IO_SetBaudRate(9600);				// bus speed

  // imposta lo stato iniziale del bus
  SetMosi(0);					// abbassa MOSI
  if(SpiMode & SPI_CPOL_HI)
  {
    SetClk(1);					// il clock riposa alto
  }
  else
  {
    SetClk(0);					// il clock riposa basso
  }

  // imposta tutte le direzioni delle 3 linee gestite
  IO_DirMaskBitWrite(MOSI,1);			// MOSI = out
  IO_DirMaskBitWrite(MISO,0);			// MISO = in
  IO_DirMaskBitWrite(CLK,1);			// CLK = out
  
  FUNCTION_LEAVE();
  return 0;					// termina con successo
}



uint32_t SPI_DataExchange(uint32_t Data)
{
  /* Invia/riceve una word sul bus SPI. */



  int		i;
  uint32_t	RecvData=0x00;



  FUNCTION_ENTER();
  MESSAGE(BUS_VERB_LEVEL,"SPI <- 0x%08lx  Nbits=%i\n",Data,Nbits);

  if(SpiMode & SPI_MSB_FIRST)
  {
    // MSB first mode
    if(SpiMode & SPI_SECOND_EDGE)		// trigger sul secondo fronte
    {
      for(i=0;i<Nbits;i++)
      {
	ToggleClk();				// inverte il clock
	SetMosi(Data & (0x0001<<(Nbits-1)));	// MSB sul bus
	Data<<=1;				// prepara il prossimo bit
	RecvData<<=1;				// fa posto al nuovo bit
	ToggleClk();				// fronte attivo
	RecvData|=(unsigned long int)GetMiso();	// legge MISO
      }
    }
    else
    {
      for(i=0;i<Nbits;i++)
      {
	SetMosi(Data & (0x0001<<(Nbits-1)));	// MSB sul bus
	ToggleClk();				// inverte il clock
	Data<<=1;				// prepara il prossimo bit
	RecvData<<=1;				// fa posto al nuovo bit
	RecvData|=(unsigned long int)GetMiso();	// legge MISO
	ToggleClk();				// fronte attivo
      }
    }
  }
  else
  {
    // LSB first mode
    if(SpiMode & SPI_SECOND_EDGE)		// trigger sul secondo fronte
    {
      for(i=0;i<Nbits;i++)
      {
	ToggleClk();				// inverte il clock
	SetMosi(Data & 0x0001);			// MSB sul bus
	Data>>=1;				// prepara il prossimo bit
	RecvData>>=1;				// fa posto al nuovo bit
	ToggleClk();				// fronte attivo
	RecvData|=((unsigned long int)GetMiso())<<(Nbits-1);	// legge MISO
      }
    }
    else
    {
      for(i=0;i<Nbits;i++)
      {
	SetMosi(Data & 0x0001);			// MSB sul bus
	ToggleClk();				// inverte il clock
	Data>>=1;				// prepara il prossimo bit
	RecvData>>=1;				// fa posto al nuovo bit
	RecvData|=((unsigned long int)GetMiso())<<(Nbits-1);	// legge MISO
	ToggleClk();				// fronte attivo
      }
    }
  }

  // assicura che il clock sia a posto
  if(SpiMode & SPI_CPOL_HI)
  {
    SetClk(1);					// il clock riposa alto
  }
  else
  {
    SetClk(0);					// il clock riposa basso
  }
  
  MESSAGE(BUS_VERB_LEVEL,"SPI -> 0x%08lx\n",RecvData);
  FUNCTION_LEAVE();

  return(RecvData);				// termina
}

/* $Source$ */

