/*
    OLED display kernel driver.
    Copyright (C) 2008 2009 2011  Alessandro Accattini


 *****************************************************************************
 *
 *
 *
 */



/***************************************************************************/
/* include vari */

#include <linux/init.h>
#include <linux/version.h>
#include <linux/errno.h>
#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/kernel.h>
#include <linux/fs.h>
#include <linux/cdev.h>				// char device
#include <asm/uaccess.h>			// user space access
#include <linux/fcntl.h>			// file flags
#include <linux/proc_fs.h>			// /proc filesystem
#include <linux/ioport.h>			// I/O level access
#include <asm/io.h>				// I/O low level handling
#include <asm/semaphore.h>			// semaphores
#include <linux/delay.h>			// ndelay, udelay, mdelay

#include "debug.h"				// debug and print macros
#include "datadev.h"				// data device operations
#include "ctrldev.h"				// ctrl device operations
#include "chardev.h"				// char device operations
#include "oledf.h"				// OLED display functions
#include "led.h"				// LED control functions
#include "spi.h"				// SPI channel
#include "io.h"					// I/O level
#include "version.h"				// driver version

/***************************************************************************/
/* define vari */

//#define DRIVER_DMESG	"OLED: "		// driver name in dmesg
#define DRIVER_NAME	"oled"			// devices name
#define DEVICES_NUM	3			// number of devices
						// 100.0: frame buffer
						// 100.1: ctrl device
						// 100.2: char display
#define PROC_FILE	"oled"			// /proc file name
#define PROC_PARENT	NULL			// create in /proc
#define ALL_STEPS	200

/***************************************************************************/
/* init and destroy functions prototype */

static int OledInit(void);
  /* Inizializza il driver, ritorna 0 in caso di successo. */

static void OledExit(void);
  /* Ripulisce tutto il lavoro fatto dal driver. */

void ClearSpace(int Step);
  /* Ripulisce le installazioni partendo dall'ultimo Step riuscito. */

/***************************************************************************/
/* file handling functions prototype */

int DeviceOpen(struct inode *Inode,struct file *Filp);
  /* Open method, inizializza il device sull'apertura del file. */

int DeviceClose(struct inode *Inode,struct file *Filp);
  /* Close method, dealloca tutto quello che ha allocato la open. */

/***************************************************************************/
/* proc handling functions prototype */

int ProcRead(char *Page,char **Start,off_t Off,int Count,int *Eof,void *Data);
  /* Proc read method, restituisce lo stato del device. */

/***************************************************************************/
/* hardware related functions prototype */

/***************************************************************************/
/* strutture */

/***************************************************************************/
/* variabili */

dev_t			DataDevice;		// first mapped device
dev_t			CtrlDevice;		// second mapped device
dev_t			CharDevice;		// third mapped device
struct file_operations	DataFops={
					.owner=		THIS_MODULE,
					.open=		DeviceOpen,
					.release=	DeviceClose,
					.read=		DataRead,
					.write=		DataWrite,
//					.ioctl=		DataIoctl,
					.llseek=	DataSeek};
struct file_operations	CtrlFops={
					.owner=		THIS_MODULE,
					.open=		DeviceOpen,
					.release=	DeviceClose,
					.read=		CtrlRead,
					.write=		CtrlWrite,
//					.ioctl=		CtrlIoctl,
					.llseek=	CtrlSeek};
struct file_operations	CharFops={
					.owner=		THIS_MODULE,
					.open=		DeviceOpen,
					.release=	DeviceClose,
					.read=		CharRead,
					.write=		CharWrite,
//					.ioctl=		CharIoctl,
					.llseek=	CharSeek};
struct cdev		DataCdev;		// cdev structure
struct cdev		CtrlCdev;		// cdev structure
struct cdev		CharCdev;		// cdev structure

/***************************************************************************/
/* parametri */

int			Major=100;		// major number
int			Minor=0;		// minor number
int			Hard=TRITON_REV_3;	// hardware type

/***************************************************************************/
/* dichiarazione funzioni e parametri */

MODULE_LICENSE("GPL");
MODULE_AUTHOR("Alessandro Accattini");

module_init(OledInit);
module_exit(OledExit);

module_param(Major,int,S_IRUGO);
module_param(Minor,int,S_IRUGO);
module_param(Hard,int,S_IRUGO);

/***************************************************************************/
/* funzioni */

static int OledInit(void)
{
  /* Inizializza il driver, ritorna 0 in caso di successo. */



  int	InitStep=0;			// ultimo passaggio riuscito
  int	Err;				// error code
  int 	i;



  PINFO("OLED display driver version " VERSION "\n");
  PINFO("Selected hardware is: ");
  switch(Hard)
  {
    case TRITON_REV_1: 
      PINFO("TRITON REV 1\n");
    break;
    case TRITON_REV_2: 
      PINFO("TRITON REV 2\n");
    break;
    case TRITON_REV_3: 
      PINFO("TRITON REV 3\n");
    break;
    default: 
      PINFO("UNKNOWN\n");
    break;
  }

  /* inizializza le strutture interne al driver */
  sema_init(&HardSem,1);		// inizializza il semaforo per l'HW

  /* alloca i device numbers */
  if(Major==0)
  {
    /* alloca dinamicamente major e minor numbers */
    PINFO("allocating dinamically major and minor numbers\n");
    Err=alloc_chrdev_region(&DataDevice,Minor,DEVICES_NUM,DRIVER_NAME);
    if(Err)
    {
      ClearSpace(InitStep);		// ripulisce l'ambiente
      return(Err);			// termina con errore
    }
    CtrlDevice=MKDEV(MAJOR(DataDevice),MINOR(DataDevice)+1);
  }
  else
  {
    /* registra i devices staticamente */
    DataDevice=MKDEV(Major,Minor);	// crea il primo device occupato
    CtrlDevice=MKDEV(Major,Minor+1);	// crea il secondo device occupato
    CharDevice=MKDEV(Major,Minor+2);	// crea il terzo device occupato
    Err=register_chrdev_region(DataDevice,DEVICES_NUM,DRIVER_NAME);
    if(Err)
    {
      ClearSpace(InitStep);		// ripulisce l'ambiente
      return(Err);			// termina con errore
    }
  }
  ++InitStep;				// step successivo

  /* visualizza i devices registrati */
  PINFO("devices registered as: %s\n",DRIVER_NAME);
  for(i=0;i<DEVICES_NUM;++i)
    PINFO("major: %i minor: %i\n",MAJOR(DataDevice),MINOR(DataDevice)+i);

  /* crea un'entry nel /proc filesystem */
  if(create_proc_read_entry(PROC_FILE,0,PROC_PARENT,ProcRead,NULL)==NULL)
  {
    ClearSpace(InitStep);		// ripulisce l'ambiente
    return(-1);				// termina con errore
  }
  ++InitStep;				// step successivo

  /* acquisisce ed inizializza l'I/O fisico */
  PINFO("init I/O ports\n");
  if(InitIO(Hard))
  {
    PINFO("initialization error\n");
    ClearSpace(InitStep);		// ripulisce l'ambiente
    return(-1);				// termina con errore
  }
  ++InitStep;				// step successivo

  /* inzializza la porta SPI e il display driver */
  SpiInit();				// inizializza l'SPI
  OledInitDisplay();			// inizializza il display
  LedInit();				// inizializza i LED
  ++InitStep;				// step successivo

  /**********************************************************************/
  /* WARNING:								*/
  /* after cdev_add call registered device is 'live'.			*/
  /* We need one cdev struct for each char device implemented.		*/
  /**********************************************************************/
  
  /* inizializza e collega la cdev struct del data device */
  cdev_init(&DataCdev,&DataFops);	// struct init
  DataCdev.owner=THIS_MODULE;		// imposta l'owner
  Err=cdev_add(&DataCdev,DataDevice,1);
  if(Err)
  {
    ClearSpace(InitStep);		// ripulisce l'ambiente
    return(Err);			// termina con errore
  }
  ++InitStep;				// step successivo

  /* inizializza e collega la cdev struct del ctrl device */
  cdev_init(&CtrlCdev,&CtrlFops);	// struct init
  CtrlCdev.owner=THIS_MODULE;		// imposta l'owner
  Err=cdev_add(&CtrlCdev,CtrlDevice,1);
  if(Err)
  {
    ClearSpace(InitStep);		// ripulisce l'ambiente
    return(Err);			// termina con errore
  }
  ++InitStep;				// step successivo

  /* inizializza e collega la cdev struct del char device */
  cdev_init(&CharCdev,&CharFops);	// struct init
  CharCdev.owner=THIS_MODULE;		// imposta l'owner
  Err=cdev_add(&CharCdev,CharDevice,1);
  if(Err)
  {
    ClearSpace(InitStep);		// ripulisce l'ambiente
    return(Err);			// termina con errore
  }
  ++InitStep;				// step successivo

  PINFO("...all done\n");
  return 0;				// termina con successo
}



static void OledExit(void)
{
  /* Ripulisce tutto il lavoro fatto dal driver. */



  ClearSpace(ALL_STEPS);		// elimina quanto installato
  return;				// termina
}



void ClearSpace(int Step)
{
  /* Ripulisce le installazioni partendo dall'ultimo Step riuscito. */



  switch(Step)
  {
    case 0:
      PALERT("can't register devices, major: %i minor: %i\n",
		MAJOR(DataDevice),MINOR(DataDevice));
    break;
    case ALL_STEPS:
    case 7:
      PINFO("remove char device\n");
      cdev_del(&CharCdev);		// remove cdev struct
    case 6:
      PINFO("remove ctrl device\n");
      cdev_del(&CtrlCdev);		// remove cdev struct
    case 5:
      PINFO("remove data device\n");
      cdev_del(&DataCdev);		// remove cdev struct
    case 4:
      LedDeinit();			// rilascia i LED
      OledDeinitDisplay();		// rilascia il display
    case 3:
      PINFO("release I/O space\n");
      /* rilascia lo spazio fisico nell'I/O */
      //release_region(Base,IO_PORTS);
    case 2:
      PINFO("remove /proc entry\n");
      remove_proc_entry(PROC_FILE,PROC_PARENT);
    case 1:
      PINFO("unregister devices\n");
      unregister_chrdev_region(DataDevice,DEVICES_NUM);
    break;
  }
  PINFO("driver completely removed.\n");
  return;				// termina
}



int DeviceOpen(struct inode *Inode,struct file *Filp)
{
  /* Open method, inizializza il device sull'apertura del file. */
  
  
  
  PDEBUG("DeviceOpen()\n");
  Filp->f_pos=0;			// reset file position
  return 0;				// termina con successo
}



int DeviceClose(struct inode *Inode,struct file *Filp)
{
  /* Close method, dealloca tutto quello che ha allocato la open. */



  PDEBUG("DeviceClose()\n");
  return 0;				// termina con successo
}



int ProcRead(char *Page,char **Start,off_t Off,int Count,int *Eof,void *Data)
{
  /* Proc read method, restituisce lo stato del device. */



  char	*KeyStat;			// stato del tasto
  int	PageLen=0;			// data page len



  /* debug info */
  PDEBUG("ProcRead(Count=%li,Off=%li)\n",
		(unsigned long int)Count,(unsigned long int)Off);

  /* crea la pagina con i dati */
  if(down_interruptible(&HardSem))	// se non ottiene il semaforo
    return(-ERESTARTSYS);		// termina immediatamente con errore
  PageLen+=sprintf(&Page[PageLen],"OLED display driver version " VERSION "\n");
  if(!Get_KEY1())
    KeyStat="pressed";
  else
    KeyStat="released";
  PageLen+=sprintf(&Page[PageLen],"KEY1: %s\n",KeyStat);
  if(!Get_KEY2())
    KeyStat="pressed";
  else
    KeyStat="released";
  PageLen+=sprintf(&Page[PageLen],"KEY2: %s\n",KeyStat);
  if(!Get_KEY3())
    KeyStat="pressed";
  else
    KeyStat="released";
  PageLen+=sprintf(&Page[PageLen],"KEY3: %s\n",KeyStat);
  if(Get_SUSPEND())
    KeyStat="suspended";
  else
    KeyStat="not suspended";
  PageLen+=sprintf(&Page[PageLen],"SUSPEND: %s\n",KeyStat);
  PageLen+=sprintf(&Page[PageLen],"CPU: %i\n",GetBoardPosition());
  *Eof=1;				// segnala end of file
  up(&HardSem);				// libera il semaforo
  return(PageLen);			// termina senza dati
}

/* $Source: /users/CVS/oled_linux/kdriver.c,v $ */

