/*
    OLED display handling functions.
    Copyright (C) 2008  Alessandro Accattini


 *****************************************************************************
 *
 *
 *
 */



/***************************************************************************/
/* include vari */

#include <linux/module.h>
#include <linux/delay.h>			// ndelay, udelay, mdelay
#include <linux/timer.h>			// kernel software timers
#include <asm/semaphore.h>			// semaphores

#include "debug.h"				// debug and print macros
#include "spi.h"				// SPI channel
#include "io.h"					// I/O level access
#include "lcd_font.h"				// font dei caratteri 6x8

/***************************************************************************/
/* define interni alla libreria */

/* display commands */
#define SET_COLUMN_ADDRESS        0x15
#define SET_ROW_ADDRESS           0x75
#define SET_CONTRAST_CONTROL_REG  0x81
#define SET_CURRENT_RANGE_Q       0x84
#define SET_CURRENT_RANGE_H       0x85
#define SET_CURRENT_RANGE_F       0x86
#define SET_RE_MAP                0xA0
#define SET_DISPLAY_START_LINE    0xA1
#define SET_DISPLAY_OFFSET        0xA2
#define SET_DISPLAY_MODE_NORM     0xA4
#define SET_DISPLAY_MODE_ALL_ON   0xA5
#define SET_DISPLAY_MODE_ALL_OFF  0xA6
#define SET_DISPLAY_MODE_INV      0xA7
#define SET_MULTIPLEX_RATIO       0xA8
#define INITIALIZATION            0xAD
#define DISPLAY_ON                0xAF
#define DISPLAY_OFF               0xAE
#define OPTIMIZE_ROW_BRIGHTNESS   0xB0
#define DISPLAY_NOP               0xE3

#define OLED_COLS		21		// numero di colonne testo
#define OLED_ROWS		16		// numero di righe testo
#define FONT_W			6		// larghezza char
#define FONT_L			8		// altezza char

#define BYTES_LINE		64		// bytes per riga nel display
#define PIXEL_ROWS		128		// righe totali del display

#define FONT_COLOR		0x0f		// colore testo

#define INIT_DATA		0x00		// valore di inizializzazione

/***************************************************************************/
/* strutture contenute all'interno della libreria */

/***************************************************************************/
/* variabili interne alla libreria */

static struct timer_list	OledTimer;	// timer structure
static unsigned char	DispInit[]={INITIALIZATION,0x02};
static unsigned char	DispOn[]={DISPLAY_ON};
static unsigned char	DispRemap[]={SET_RE_MAP,0x52};
static unsigned char	DispRow[]={SET_ROW_ADDRESS,0,127};
static unsigned char	DispCol[]={SET_COLUMN_ADDRESS,0,63};
static unsigned char	DispGotoX[]={SET_COLUMN_ADDRESS,0,63};
static unsigned char	DispGotoY[]={SET_ROW_ADDRESS,0,127};

int			CursorCol=0;		// colonna cursore
int			CursorPtr=64*FONT_L*(OLED_ROWS-1);

/***************************************************************************/
/* funzioni interne alla libreria */

static void OledScrollUp(void);
  /* Scrolla lo schemo verso l'alto di una riga. */

static void OledPutCharRow(unsigned char RowShape,int FramePtr);
  /* Inserisce nel frame buffer una riga del font. */

static void OledPutChar(char Letter);
  /* Stampa un char alla posizione attuale del cursore, il cursore avanza
     di una posizione, se il cursore esce dalla riga viene riportato a
     capo e viene scrollato tutto il testo. */

static void OledTimeElapsed(unsigned long Data);
  /* Spegne il display allo scadere del tempo programmato. */

/***************************************************************************/
/* define pubblici */

#define BUFFER_SIZE	BYTES_LINE*PIXEL_ROWS	// dimensione frame buffer

/***************************************************************************/
/* strutture pubbliche */

/***************************************************************************/
/* variabili pubbliche */

struct semaphore	HardSem;		// hardware access semaphore
unsigned char		DataBuffer[BUFFER_SIZE];	// frame buffer

/***************************************************************************/
/* funzioni pubbliche */

void OledInitDisplay(void);
  /* Inizializza il display driver. */

void OledDeinitDisplay(void);
  /* Elimina le risorse allocate dal display driver. */

void OledSendCommand(unsigned char *Cmd,unsigned long int CmdLen);
  /* Invia un array di command bytes al display driver. */

void OledSendData(unsigned char *Data,unsigned long int DataLen);
  /* Invia un array di data bytes al display driver. */

void OledRefreshImg(void);
  /* Copia l'intero frame buffer sul diplay. */

void OledPrintStr(char *Str,unsigned long int StrLen);
  /* Stampa una stringa a video come se fosse un display alfanumerico. */

void OledTurnOnDisplay(void);
  /* Abilita l'immagine sul display e riavvia il timer per lo spegnimento. */

void OledTurnOffDisplay(void);
  /* Spegne il display e distrugge il timer per lo spegnimento. */

/***************************************************************************/
/* funzioni */

static void OledScrollUp(void)
{
  /* Scrolla lo schemo verso l'alto di una riga. */



  int	i;



  /* muove tutte le righe tranne la prima verso l'alto */
  PDEBUG("%s: scrolling up\n",__FUNCTION__);
  for(i=0;i<BYTES_LINE*(PIXEL_ROWS-FONT_L);++i)
    DataBuffer[i]=DataBuffer[i+(BYTES_LINE*FONT_L)];	// copy data image

  /* ripulisce l'ultima riga del display */
  PDEBUG("%s: cleaning new empty line\n",__FUNCTION__);
  for(i=BYTES_LINE*(PIXEL_ROWS-FONT_L);i<BYTES_LINE*PIXEL_ROWS;++i)
    DataBuffer[i]=INIT_DATA;			// clear data image
  return;					// termina
}



static void OledPutCharRow(unsigned char RowShape,int FramePtr)
{
  /* Inserisce nel frame buffer una riga del font. */



  int		i;
  unsigned char	Color;				// color shape

  
  
  RowShape<<=2;					// riposiziona i bit
  for(i=0;i<3;++i)
  {
    Color=0;					// inzializza il colore
    if(RowShape&0x80)
      Color|=FONT_COLOR<<4;			// colora il pixel SX
    if(RowShape&0x40)
      Color|=FONT_COLOR;			// colora il pixel DX
    DataBuffer[FramePtr+i]=Color;		// inserisce nel frame buffer
    RowShape<<=2;				// shifta la sagoma
  }
  return;					// termina
}



static void OledPutChar(char Letter)
{
  /* Stampa un char alla posizione attuale del cursore, il cursore avanza
     di una posizione, se il cursore esce dalla riga viene riportato a
     capo e viene scrollato tutto il testo. */



  int	i;



  switch(Letter)
  {
    case '\n':
      OledScrollUp();				// scrolla il video
    case '\r':
      CursorCol=0;				// cursore sulla colonna 0
      CursorPtr=64*FONT_L*(OLED_ROWS-1);	// colonna 0 ultima riga
    break;
    default:
      for(i=0;i<FONT_L;++i)
        OledPutCharRow(LcdFont[((int)Letter)*FONT_L+i],CursorPtr+i*64);
      CursorCol++;				// sposta il cursore avanti
      CursorPtr+=FONT_W/2;
      if(CursorCol>=OLED_COLS)
      {
	OledScrollUp();				// scrolla il video
	CursorCol=0;				// cursore sulla colonna 0
	CursorPtr=64*FONT_L*(OLED_ROWS-1);	// colonna 0 ultima riga
      }
    break;
  }
  return;					// termina
}



static void OledTimeElapsed(unsigned long Data)
{
  /* Spegne il display allo scadere del tempo programmato. */



  PDEBUG("%s: display auto-shut-off\n",__FUNCTION__);
  DisableOled();				// disabilita il display
  return;					// termina
}



void OledInitDisplay(void)
{
  /* Inizializza il display driver. */



  int		i;
  


  PDEBUG("%s()\n",__FUNCTION__);		// debug call info
  
  /* display hardware reset */
  PDEBUG("%s: hardware reset\n",__FUNCTION__);	// debug info
  Set_OLED_RES(0);				// abbassa RES
  mdelay(12);
  Set_OLED_RES(1);				// alza RES
  mdelay(12);

  /* display power up sequence */
  PDEBUG("%s: display power up\n",__FUNCTION__);	// debug info
  OledSendCommand(DispInit,2);
  OledSendCommand(DispOn,1);
  OledSendCommand(DispRemap,2);

  /* clear display */
  PDEBUG("%s: clear display\n",__FUNCTION__);	// debug info
  OledSendCommand(DispOn,1);
  OledSendCommand(DispRow,3);
  OledSendCommand(DispCol,3);
  
  for(i=0;i<BUFFER_SIZE;++i)
    DataBuffer[i]=INIT_DATA;			// clear image
  //OledSendData(DataBuffer,BUFFER_SIZE);		// send data
  OledRefreshImg();				// copia tutto il frame buffer

  /* inizializza il timer per lo spegnimento */
  init_timer(&OledTimer);			// inizializza la struttura
  OledTimer.function=OledTimeElapsed;		// imposta la funzione chiamata
  OledTimer.expires=jiffies+(20*HZ);		// imposta la scadenza
  add_timer(&OledTimer);			// avvia il timer
  return;					// termina
}



void OledDeinitDisplay(void)
{
  /* Elimina le risorse allocate dal display driver. */
  
  
  
  del_timer_sync(&OledTimer);			// distrugge il timer
  return;					// termina
}



void OledSendCommand(unsigned char *Cmd,unsigned long int CmdLen)
{
  /* Invia un array di command bytes al display driver. */
  
  
  
  unsigned long int	i;
  
  
  
  PDEBUG("%s(*Cmd,%lu) Cmd[]=0x%02x,0x%02x,0x%02x,0x%02x,0x%02x\n",
  	__FUNCTION__,CmdLen,(unsigned int)Cmd[0],(unsigned int)Cmd[1],
	(unsigned int)Cmd[2],(unsigned int)Cmd[3],(unsigned int)Cmd[4]);
  Set_OLED_DC(0);				// command mode
  Set_OLED_CS(0);				// abbassa CS
  for(i=0;i<CmdLen;++i)
    SpiExchange(Cmd[i]);			// invia un command byte
  Set_OLED_CS(1);				// alza CS
  Set_OLED_DC(1);				// data mode
  return;					// termina
}



void OledSendData(unsigned char *Data,unsigned long int DataLen)
{
  /* Invia un array di data bytes al display driver. */



  unsigned long int	i;



  PDEBUG("%s(*Data,%lu)\n",__FUNCTION__,DataLen);
  Set_OLED_DC(1);				// data mode
  Set_OLED_CS(0);				// abbassa CS
  for(i=0;i<DataLen;++i)
    SpiExchange(Data[i]);			// invia un data byte
  Set_OLED_DC(0);				// command mode
  SpiExchange(DISPLAY_NOP);			// termina il data stream
  Set_OLED_CS(1);				// alza CS
  Set_OLED_DC(1);				// data mode
  return;					// termina
}



void OledRefreshImg(void)
{
  /* Copia l'intero frame buffer sul diplay. */
  
  
  
  OledSendCommand(DispGotoX,3);			// riposiziona il pennello
  OledSendCommand(DispGotoY,3);
  OledSendData(DataBuffer,BUFFER_SIZE);		// copy frame buffer
  return;					// termina
}



void OledPrintStr(char *Str,unsigned long int StrLen)
{
  /* Stampa una stringa a video come se fosse un display alfanumerico. */



  unsigned long int	i;



  for(i=0;i<StrLen;++i)
    OledPutChar(Str[i]);			// stampa un char
  OledRefreshImg();				// rinfresca l'immagine
  return;					// termina
}



void OledTurnOnDisplay(void)
{
  /* Abilita l'immagine sul display e riavvia il timer per lo spegnimento. */



  PDEBUG("%s: display turn-on\n",__FUNCTION__);
  EnableOled();					// abilita il display
  mod_timer(&OledTimer,jiffies+(20*HZ));	// riarma il timer
  return;					// termina
}



void OledTurnOffDisplay(void)
{
  /* Spegne il display e distrugge il timer per lo spegnimento. */



  PDEBUG("%s: display shut-off\n",__FUNCTION__);
  if(timer_pending(&OledTimer))			// se il timer e' schedulato
    del_timer_sync(&OledTimer);			// distrugge il timer
  DisableOled();				// disabilita il display
  return;					// termina
}

/* $Source: /users/CVS/oled_linux/oledf.c,v $ */

