/*
    Universal hardware control environment.
    Copyright (C) 2010 2011 2012 2013 2014 2016 2020  Alessandro Accattini

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


 *****************************************************************************
 *
 *
 * $Header$
 *
 *
 *
 * REVISION HISTORY:
 * $Log$
 */


/***************************************************************************/
/* include vari */

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <time.h>			// execution time measurement
#include "commands/commands.h"		// commands lib
#include "jimtcl/jim.h"			// TCL interpreter

/***************************************************************************/
/* define vari */

#define VER_MAJOR		0	// code version
#define VER_MINOR		12


  // flag vari
#define FLAG_EXEC_TIME		0x01
#define FLAG_INTERACTIVE	0x02
#define FLAG_COMMAND_PRINT	0x04

/***************************************************************************/
/* macro */

#define PDEBUG(fmt,args...)	printf(fmt,## args)

/***************************************************************************/
/* strutture */

/***************************************************************************/
/* costanti */

static const char Licenza[]=
	"struttura version %i.%02i, Copyright (C) 2010-2020  "
	"Alessandro Accattini\n\n";
static const char Help[]=
	"Usage: struttura [OPTIONS] [file_1] [file_2]...\n"
	"struttura is an environment to drive hardware devices.\n\n"
	"\t-h\tdisplay this help.\n"
	"\t-l\tlist all supported chipsets and exit.\n"
	"\t-L\tlong list of all supported chipsets.\n"
	"\t-v N\tset verbosity level.\n"
	"\t-C\tprints out the command to being executed.\n"
	"\t-I N\tforce N hardware interfaces.\n"
	"\t-P HEX\tset chipset product code.\n"
	"\t-T\tenable commands execution time measure.\n"
	"\t-V HEX\tset chipset vendor code.\n"
	"\nReport bugs to bit.trapper@gmail.com\n";

//static char	Prompt[]="struttura> ";		// prompt dei comandi

/***************************************************************************/
/* variabili */

//char	InBuf[256];				// input da tastiera

/***************************************************************************/
/* prototipi funzioni */

static void Jim_ErrorMessage(Jim_Interp *Interp);
  /* Stampa lerrore in cui e' incappato l'interprete. */

/***************************************************************************/
/* funzioni */

static void Jim_ErrorMessage(Jim_Interp *Interp)
{
  /* Stampa lerrore in cui e' incappato l'interprete. */



  Jim_MakeErrorMessage(Interp);
  fprintf(stderr,"%s\n",Jim_String(Jim_GetResult(Interp)));
  return;					// termina
}





/***************************************************************************/

int main(int argc,char **argv)
{
  Jim_Interp	*Interp;		// interprete TCL
  int		Vendor=0x0403;		// vendor code
  int		Product=0x6001;		// product code
  int		Opzione;		// per la lettura delle opzioni
  int		HardIfaces=0;		// numero interfacce hardware
  int		Error;
  unsigned int	Mode=0;			// flag vari



  // visualizza la licenza del programma
  printf(Licenza,VER_MAJOR,VER_MINOR);
  
  // legge le opzioni presenti sulla riga di comando
  while((Opzione=getopt(argc,argv,"hlLTv:CV:P:I:"))!=-1)
  {
    switch(Opzione)
    {
      case 'h':
        puts(Help);				// stampa l'help
	return 0;				// termina
//      case 'l':
//        sprintf(InBuf,"list");
//        ExecCmd(InBuf);			// stampa la lista dei chipset
//	return 0;				// termina
//      case 'L':
//        sprintf(InBuf,"list long");
//        ExecCmd(InBuf);			// stampa la lista dei chipset
//	return 0;				// termina
//      case 'v':
//        sscanf(optarg,"%i",&Verbosity);	// imposta la verbosita'
//	break;
      case 'V':
        sscanf(optarg,"%i",&Vendor);		// imposta il vendor code
	break;
      case 'P':
        sscanf(optarg,"%i",&Product);		// imposta il product code
	break;
      case 'I':
        sscanf(optarg,"%i",&HardIfaces);	// forza le interfacce hardware
	break;
//      case 'C':
//        Mode|=FLAG_COMMAND_PRINT;		// abilita stampa dei comandi
//	break;
//      case 'T':
//        Mode|=FLAG_EXEC_TIME;			// abilita tempo esecuzione
//	break;
      default:
        puts("Unknown option, use -h to see help.");
	return 0;			// termina
    }
  }

  // apre il controllo dell'interfacia hardware
  if(IO_Open(Vendor,Product,HardIfaces))	// se fallisce
    return 1;					// termina con fallimento
  putchar('\n');				// lascia una riga

  // crea l'interprete TCL
  Interp=Jim_CreateInterp();			// crea l'interprete
  Jim_RegisterCoreCommands(Interp);		// comandi standard
  if(Jim_InitStaticExtensions(Interp) != JIM_OK)	// estensioni statiche
  {
    Jim_ErrorMessage(Interp);			// stampa l'errore
  }
  if(Jim_AddConstructCommands(Interp))		// comandi custom
  {
    Jim_ErrorMessage(Interp);			// stampa l'errore
  }
  
  // processa tutti gli altri parametri passati
  if(optind >= argc)				// se non ci sono file sorgenti
  {
    // modalita' interattiva
    INTERACTIVE:
    puts("entering interactive mode...\n\n");
    Jim_InteractivePrompt(Interp);		// interprete interattivo
  }
  else
  {
    // interpreta i file passati dalla riga di comando
    do  
    {
      printf("opening file: %s\n",argv[optind]);	// file name
      if(!strcmp(argv[optind],"aa"))			// se apre il file "aa"
        goto INTERACTIVE;				// diventa interattivo
      Error=Jim_EvalFile(Interp,argv[optind++]);	// interpreta il file
      if(Error == JIM_ERR)
      {
        Jim_MakeErrorMessage(Interp);
	fprintf(stderr,"%s\n",Jim_GetString(Jim_GetResult(Interp),NULL));
      }
    }while(optind < argc);
  }
  
  // chiude e termina
  Jim_FreeInterp(Interp);			// distrugge l'interprete
  IO_Close();					// chiude l'hardware
  puts("\n\nThank you for using 'struttura'!\n");
  return 0;				// termina
}

/* $Source$ */

